from django import forms
from django.conf import settings


import hashlib

class CecaPaymentForm(forms.Form):
    # Campos del formulario
    MerchantID = forms.CharField(widget=forms.HiddenInput(), required=False)
    AcquirerBIN = forms.CharField(widget=forms.HiddenInput(), required=False)
    TerminalID = forms.CharField(widget=forms.HiddenInput(), required=False)
    Num_operacion = forms.CharField(widget=forms.HiddenInput(), required=False)
    Importe = forms.CharField(widget=forms.HiddenInput(), required=False)
    TipoMoneda = forms.CharField(widget=forms.HiddenInput(), required=False)
    Exponente  = forms.CharField(widget=forms.HiddenInput(), required=False)
    URL_OK  = forms.CharField(widget=forms.HiddenInput(), required=False)
    URL_NOK  = forms.CharField(widget=forms.HiddenInput(), required=False)
    Firma = forms.CharField(widget=forms.HiddenInput(), required=False)
    Cifrado = forms.CharField(widget=forms.HiddenInput(), required=False)
    Idioma = forms.CharField(widget=forms.HiddenInput(), required=False)
    Pago_soportado = forms.CharField(widget=forms.HiddenInput(), required=False)
    Descripcion = forms.CharField(widget=forms.HiddenInput(), required=False)
    # Pago_elegido = forms.CharField(widget=forms.HiddenInput(), required=False)
    # PAN = forms.CharField(widget=forms.HiddenInput(), required=False)
    # Caducidad = forms.CharField(widget=forms.HiddenInput(), required=False)
    # CVV2 = forms.CharField(widget=forms.HiddenInput(), required=False)
    # Referencia = forms.CharField(widget=forms.HiddenInput(), required=False)

    def __init__(self, *args, **kwargs):
        super(CecaPaymentForm, self).__init__(*args, **kwargs)

        if self.initial:
            self.settings = settings.MERCHANT_SETTINGS["ceca"]
            self.valores_por_defecto()
            self.generar_firma()


    def valores_por_defecto(self):
        self.initial['MerchantID'] = self.settings['merchant_id']
        self.initial['AcquirerBIN'] = self.settings['acquirer_bin']
        self.initial['TerminalID'] = self.settings['terminal_id']
        self.initial['Exponente'] = 2

        if not hasattr(self.initial, 'TipoMoneda'):
            self.initial['TipoMoneda'] = 978


        if not hasattr(self.initial, 'Idioma'):
            self.initial['Idioma'] = 1

        self.initial['Importe'] = int(self.initial['Importe'] * 100)
        self.initial['Cifrado'] = "SHA1"
        self.initial['Pago_soportado'] = "SSL"


        
    def generar_firma(self):
        string = "%s%s%s%s%s%s%s%s%s%s%s" % (
            self.settings['clave_encriptacion'],
            self.initial['MerchantID'],
            self.initial['AcquirerBIN'],
            self.initial['TerminalID'],
            self.initial['Num_operacion'],
            self.initial['Importe'],
            self.initial['TipoMoneda'],
            self.initial['Exponente'],
            self.initial['Cifrado'],
            self.initial['URL_OK'],
            self.initial['URL_NOK'],
        )

        self.initial['Firma'] = hashlib.sha1(string).hexdigest()



from django.conf import settings
from django.conf.urls import patterns, url
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.utils.decorators import method_decorator
from django.http import HttpResponse

from billing import Integration, IntegrationNotConfigured
from billing.signals import transaction_was_successful, transaction_was_unsuccessful

from ..forms.ceca import CecaPaymentForm
from ..models.ceca import CecaResponse

csrf_exempt_m = method_decorator(csrf_exempt)
require_POST_m = method_decorator(require_POST)

URL_LIVE = "https://pgw.ceca.es/cgi-bin/tpv"
URL_TEST = "http://tpv.ceca.es:8000/cgi-bin/tpv"

class CecaIntegration(Integration):
    display_name = "Ceca"
    template = "billing/ceca.html"
    def __init__(self):
        super(CecaIntegration, self).__init__()
        merchant_settings = getattr(settings, "MERCHANT_SETTINGS")
        if not merchant_settings or not merchant_settings.get("ceca"):
            raise IntegrationNotConfigured("The '%s' integration is not correctly "
                                           "configured." % self.display_name)

        self.url_notification = merchant_settings['ceca']['url_notificacion']

    def form_class(self):
        return CecaPaymentForm

    def generate_form(self):
        return self.form_class()(initial=self.fields)

    def get_urls(self):
        urlpatterns = patterns('',
           url("^%s$" % self.url_notification, self.notify_handler, name="ceca_notify")
        )
        return urlpatterns

    @property
    def service_url(self):
        if self.test_mode:
            return URL_TEST
        return URL_LIVE


    @csrf_exempt_m
    @require_POST_m
    def notify_handler(self, request):
        post_data = request.POST.copy()
        data = {}

        resp_fields = {
            'MerchantID': 'merchant_id',
            'AcquirerBIN': 'acquirer_bin',
            'TerminalID': 'terminal_id',
            'Num_operation': 'num_operacion',
            'Importe': 'importe',
            'TipoMoneda': 'tipo_moneda',
            'Referencia': 'referencia',
            'Firma': 'firma',
            'Num_aut': 'numero_autorizacion',
            'Idioma': 'idioma',
            'Pais': 'pais',
            'Descripcion': 'descripcion',
        }

        for (key, val) in resp_fields.iteritems():
            data[val] = post_data.get(key, '')

        try:
            resp = CecaResponse.objects.create(**data)
            # TODO: Make the type more generic
            transaction_was_successful.send(sender=self.__class__, type="purchase", response=resp)
            status = "$*$OKY$*$"
        except:
            transaction_was_unsuccessful.send(sender=self.__class__, type="purchase", response=post_data)
            status = "FAILURE"

        return HttpResponse(status)



from django.db import models


class CecaResponse(models.Model):
    # merchant details
    merchant_id = models.CharField(max_length=25)
    acquirer_bin = models.CharField(max_length=25, blank=True, null=True)
    terminal_id = models.CharField(max_length=25, blank=True, null=True)

    # purchase details sent by merchant
    num_operacion = models.CharField(max_length=50, blank=True, null=True)
    importe  = models.CharField(max_length=50, blank=True, null=True)
    tipo_moneda   = models.CharField(max_length=50, blank=True, null=True)
    referencia  = models.CharField(max_length=50, blank=True, null=True)
    firma  = models.CharField(max_length=50, blank=True, null=True)
    numero_autorizacion  = models.CharField(max_length=50, blank=True, null=True)
    idioma  = models.CharField(max_length=50, blank=True, null=True)
    pais  = models.CharField(max_length=50, blank=True, null=True)
    descripcion  = models.CharField(max_length=50, blank=True, null=True)

    class Meta:
        app_label = __name__.split(".")[0]
